@@ -1,9 +1,9 @@
 //**********************************************************************
 //**********************************************************************
 //
-//  RANDOM SURVIVAL FOREST 3.0.1
+//  RANDOM SURVIVAL FOREST 3.2.0
 //
-//  Copyright 2007, Cleveland Clinic
+//  Copyright 2008, Cleveland Clinic Foundation
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -55,9 +55,12 @@
 //**********************************************************************
 
 extern int      *_seed_;
+extern double   *_performance_;
 extern uint     *_leafCount_;
-extern uint      _mup;
+extern uint     *_varUsed_;
+extern uint      _opt;
 extern uint      _splitRule;
+extern uint      _imputeSize;
 extern uint      _forestSize;
 extern uint      _minimumDeathCount;
 extern uint      _randomCovariateCount;
@@ -74,12 +77,17 @@ extern double   *_fxData;
 extern uint      _timeInterestSize;
 extern double   *_timeInterest;
 extern SEXP      _sexp_xType;
+extern uint      _intrPredictorSize;
+extern uint     *_intrPredictor;
+extern uint     *_intrObservation;
 extern char    **_xType;
 extern double  **_observation;
 extern double  **_fobservation;
 extern double   *_masterTime;
 extern uint     *_masterTimeIndex;
 extern uint      _masterTimeSize;
+extern char     *_importanceFlag;
+extern char      _mTimeIndexFlag; 
 extern uint     *_mRecordMap;
 extern uint     *_fmRecordMap;
 extern uint      _mRecordSize;
@@ -98,9 +106,25 @@ extern double   *_mStatus;
 extern double   *_fmStatus;
 extern double   *_mTime;
 extern double   *_fmTime;
+extern double  **_oobEnsemblePtr;
+extern double  **_fullEnsemblePtr;
+extern double   *_ensembleRun;
+extern double  **_vimpEnsembleRun;  
+extern uint     *_oobEnsembleDen;
+extern uint     *_fullEnsembleDen;
+extern double   *_sImputeStatusPtr;
+extern double   *_sImputeTimePtr;
+extern double  **_sImputePredictorPtr;
+extern double   *_sOOBImputeStatusPtr;
+extern double   *_sOOBImputeTimePtr;
+extern double  **_sOOBImputePredictorPtr;
+extern uint   **_varUsedPtr;
 extern int      *_seed1Ptr;
 extern int      *_seed2Ptr;
 extern Node    **_nodeMembership;
 extern uint     *_bootMembershipIndex;
 extern char     *_bootMembershipFlag;
+extern uint     *_oobSampleSize;
 extern Node    **_fnodeMembership;
+extern char     *_fbootMembershipFlag;
+extern uint     *_foobSampleSize;
