@@ -1,9 +1,9 @@
 //**********************************************************************
 //**********************************************************************
 //
-//  RANDOM SURVIVAL FOREST 3.0.1
+//  RANDOM SURVIVAL FOREST 3.2.0
 //
-//  Copyright 2007, Cleveland Clinic
+//  Copyright 2008, Cleveland Clinic Foundation
 //
 //  This program is free software; you can redistribute it and/or
 //  modify it under the terms of the GNU General Public License
@@ -59,28 +59,18 @@ void updateTraceFlag(char reset);
 char getBestSplit(Node    *parent,
                   uint    *splitParameterMax,
                   uint    *splitValueMax,
-                  double **masterSplit,
-                  uint   **masterSplitOrder);
-char makeTree(uint     b,
+                  double **masterSplit);
+char makeTree(char     imputeFlag,
+              uint     b,
               Node    *parent,
-              double **masterSplit,
-              uint   **masterSplitOrder,
-              char     mTimeIndexFlag);
+              double **masterSplit);
 char forkAndUpdate(uint  *leafCount,
                    Node  *parent,
                    uint   splitParameter,
                    uint   splitValueIndex,
                    double splitValue);
-char bootstrap(uint     mode,
-               uint     b,
-               Node    *rootPtr,
-               uint    *oobSampleSize,
-               double **masterSplit,
-               uint    *masterSplitSize,
-               uint   **masterSplitOrder,
-               char   **mRecordBootFlag);
 SEXP rsfGrow(SEXP traceFlag,
-             SEXP mup,
+             SEXP opt,
              SEXP seedPtr,
              SEXP splitRule,
              SEXP randomCovariateCount,
@@ -94,9 +84,10 @@ SEXP rsfGrow(SEXP traceFlag,
              SEXP timeInterestSize,
              SEXP timeInterest,
              SEXP randomCovariateWeight,
-             SEXP xType);
+             SEXP xType,
+             SEXP reimputeSize);
 SEXP rsfPredict(SEXP traceFlag,
-                SEXP mup,
+                SEXP opt,
                 SEXP seedPtr,  
                 SEXP forestsize,
                 SEXP observationSize,
@@ -116,4 +107,4 @@ SEXP rsfPredict(SEXP traceFlag,
                 SEXP spltPT,
                 SEXP seedVector,
                 SEXP xType);
-SEXP rsf(uint mode, uint traceFlag);
+SEXP rsf(char mode, uint traceFlag);
